==> gnudatalanguage_0.9~rc1.orig.tar.gz: gdl-0.9rc1/src/datalistt.hpp <==
/***************************************************************************
                        datalistt.hpp  -  holds (local and global) variables
                             -------------------
    begin                : July 22 2002
    copyright            : (C) 2002 by Marc Schellens
    email                : m_schellens@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DATALISTT_HPP_
#define DATALISTT_HPP_

#include "basegdl.hpp"

class DataListT
{
  std::vector<BaseGDL**> env; // holds the variables
  std::vector<BaseGDL*>  loc; // for local data

public:
  DataListT(): env(), loc() {}
  ~DataListT()
  {
    for( SizeT i=0; i<loc.size(); i++)
      delete loc[i];
  }

  bool InLoc( BaseGDL** p) const
  {
    return ( !loc.empty() && p >= &loc.front() && p <= &loc.back());
  }

  bool Contains( BaseGDL* p) const
  {
    for( SizeT i=0; i<loc.size(); i++)
      {
	if( loc[i] == p) return true;
	if( env[i] != NULL && *env[i] == p) return true;
      }
    return false;
  }

  void RemoveLoc( BaseGDL* p) 
  {
    for( SizeT i=0; i<loc.size(); i++)
      if( loc[i] == p)
	{
	  loc[i] = NULL;
	  return;
	}
  }

  void push_back( BaseGDL* p)
  {
    loc.push_back(p);
    env.push_back(NULL);
  }
  void push_back( BaseGDL** pp)
  {
    loc.push_back(NULL);
    env.push_back(pp);
  }
  SizeT size() { return loc.size();}
  void reserve( SizeT s)
  {
    env.reserve( s);
    loc.reserve( s);
  }
  void resize( SizeT s)
  {
    env.resize( s, NULL);
    loc.resize( s, NULL);
  }

  BaseGDL*& operator[]( const SizeT ix)
  {
    if( env[ ix] != NULL) return *env[ ix];
    return loc[ ix];
  }
  void Clear( SizeT ix)
  {
    loc[ ix]=NULL;
    env[ ix]=NULL;
  }
  void Reset( SizeT ix, BaseGDL* p)
  {
    if( loc[ ix] != NULL) delete loc[ ix];
    loc[ ix]=p;
    env[ ix]=NULL;
  }
  void Reset( SizeT ix, BaseGDL** pp)
  {
    if( loc[ ix] != NULL) delete loc[ ix];
    loc[ ix]=NULL;
    env[ ix]=pp;
  }
  void Set( SizeT ix, BaseGDL* p)
  {
    loc[ ix]=p;
    env[ ix]=NULL;
  }
  void Set( SizeT ix, BaseGDL** pp)
  {
    loc[ ix]=NULL;
    env[ ix]=pp;
  }
  bool IsSet( SizeT ix)
  {
    return (loc[ ix] != NULL || env[ ix] != NULL);
  }
  BaseGDL* Grab( SizeT ix)
  {
    BaseGDL* ret;
    if( loc[ ix] != NULL) 
      {
	ret=loc[ ix];
	loc[ ix]=NULL;
	return ret;
      }
    if( env[ ix] != NULL) return (*env[ ix])->Dup();
    return NULL;
 }

  // finds the local variable pp points to
  int FindLocal( BaseGDL** pp)
  {
    for( SizeT i=0; i<loc.size(); i++)
      if( &loc[i] == pp) return static_cast<int>(i);
    return -1;
  }

  // finds the gloabl variable pp
  int FindGlobal( BaseGDL** pp)
  {
    for( SizeT i=0; i<env.size(); i++)
      if( env[i] == pp) return static_cast<int>(i);
    return -1;
  }

  BaseGDL** GetPtrTo( BaseGDL* p)
  {
    for( SizeT i=0; i<env.size(); i++)
      {
	if( loc[i] == p) return &loc[i];
	if( env[i] != NULL && *env[i] == p) return env[i];
      }
    return NULL;
  }

  BaseGDL* Loc( SizeT ix)
  {
    return loc[ ix];
  }
  BaseGDL** Env( SizeT ix)
  {
    return env[ ix];
  }
};

#endif

==> gnudatalanguage_0.9.1.orig.tar.gz: gdl-0.9.1/src/datalistt.hpp <==
/***************************************************************************
                        datalistt.hpp  -  holds (local and global) variables
                             -------------------
    begin                : July 22 2002
    copyright            : (C) 2002 by Marc Schellens
    email                : m_schellens@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DATALISTT_HPP_
#define DATALISTT_HPP_

#include "basegdl.hpp"

struct EnvType
{
  EnvType( BaseGDL* pIn, BaseGDL** ppIn): p(pIn), pp(ppIn) {}
  bool IsSet() { return (p != NULL || pp != NULL);}
  BaseGDL* p;
  BaseGDL** pp;
};

class DataListT
{
  std::vector<EnvType> env; // holds the variables

public:
  DataListT(): env() {}
  ~DataListT()
  {
    for( SizeT i=0; i<env.size(); i++)
      delete env[i].p;
  }

  bool InLoc( BaseGDL** p) const
  {
    return ( !env.empty() && p >= &env.front().p && p <= &env.back().p);
  }

  bool Contains( BaseGDL* p) const
  {
    for( SizeT i=0; i<env.size(); i++)
      {
	if( env[i].p == p) return true;
	if( env[i].pp != NULL && *(env[i].pp) == p) return true;
      }
    return false;
  }

  void RemoveLoc( BaseGDL* p) 
  {
    for( SizeT i=0; i<env.size(); i++)
      if( env[i].p == p)
	{
	  env[i].p = NULL;
	  return;
	}
  }

  void push_back( BaseGDL* p)
  {
    env.push_back(EnvType(p,NULL));
  }
  void push_back( BaseGDL** pp)
  {
    env.push_back(EnvType(NULL,pp));
  }

  void AddOne()
  {
    env.push_back(EnvType(NULL,NULL));
  }

  void pop_back()
  {
    env.pop_back();
  }

  SizeT size() const { return env.size();}
  void reserve( SizeT s)
  {
    env.reserve( s);
  }
  void resize( SizeT s)
  {
    env.resize( s, EnvType(NULL,NULL));
  }

  BaseGDL*& operator[]( const SizeT ix)
  {
    if( env[ ix].pp != NULL) return *env[ ix].pp;
    return env[ ix].p;
  }
  void Clear( SizeT ix)
  {
    env[ ix]=EnvType(NULL,NULL);
  }
  void Reset( SizeT ix, BaseGDL* p)
  {
    if( env[ ix].p != NULL) delete env[ ix].p;
    env[ ix]=EnvType(p,NULL);
  }
  void Reset( SizeT ix, BaseGDL** pp)
  {
    if( env[ ix].p != NULL) delete env[ ix].p;
    env[ ix]=EnvType(NULL,pp);
  }
  void Set( SizeT ix, BaseGDL* p)
  {
    env[ ix]=EnvType(p,NULL);
  }
  void Set( SizeT ix, BaseGDL** pp)
  {
    env[ ix]=EnvType(NULL,pp);
  }
  bool IsSet( SizeT ix)
  {
    return env[ix].IsSet(); //(env[ ix].p != NULL || env[ ix].pp != NULL);
  }
  BaseGDL* Grab( SizeT ix)
  {
    BaseGDL* ret;
    if( env[ ix].p != NULL) 
      {
	ret=env[ ix].p;
	env[ ix].p=NULL;
	return ret;
      }
    if( env[ ix].pp != NULL) return (*env[ ix].pp)->Dup();
    return NULL;
 }

  // finds the local variable pp points to
  int FindLocal( BaseGDL** pp)
  {
    for( SizeT i=0; i<env.size(); i++)
      if( &env[i].p == pp) return static_cast<int>(i);
    return -1;
  }

  // finds the gloabl variable pp
  int FindGlobal( BaseGDL** pp)
  {
    for( SizeT i=0; i<env.size(); i++)
      if( env[i].pp == pp) return static_cast<int>(i);
    return -1;
  }

  BaseGDL** GetPtrTo( BaseGDL* p)
  {
    for( SizeT i=0; i<env.size(); i++)
      {
	if( env[i].p == p) return &env[i].p;
	if( env[i].pp != NULL && *env[i].pp == p) return env[i].pp;
      }
    return NULL;
  }

  BaseGDL* Loc( SizeT ix)
  {
    return env[ ix].p;
  }
  BaseGDL** Env( SizeT ix)
  {
    return env[ ix].pp;
  }
};

#endif
